#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Suggested updates to obspy.io.reftek.packet:
- Change type of EH_PAYLOAD['sampling_rate'] from int to float.
- Add eh_et_info method to EHPacket class. This method compiles EH and ET info
  and returns a list of strings used to write the rt2ms log file.

Maeva Pourpoint IRIS/PASSCAL
"""

import obspy.io.reftek.packet as obspy_rt130_packet

from obspy import UTCDateTime
from obspy.io.reftek.util import (_decode_ascii,
                                  _parse_long_time,
                                  _16_tuple_ascii,
                                  _16_tuple_int,
                                  _16_tuple_float)
from sohstationviewer.model.reftek.from_rt2ms.soh_packet import Packet


class Reftek130UnpackPacketError(ValueError):
    pass


eh_et_payload_last_field_start = 88
eh_et_payload_last_field_size = 16

# The payload start is based on the start of the payload, so we have to add 24
# to compensate for the size of the header and extended header.
eh_et_payload_end_in_packet = eh_et_payload_last_field_start + eh_et_payload_last_field_size + 24

# name, offset, length (bytes) and converter routine for EH/ET packet payload
EH_PAYLOAD = {
    "station_name_extension": (35, 1, _decode_ascii),
    "station_name": (36, 4, _decode_ascii),
    "sampling_rate": (64, 4, float),
    "trigger_time": (72, 16, _parse_long_time),
    "first_sample_time": (eh_et_payload_last_field_start, eh_et_payload_last_field_size, _parse_long_time),
}
obspy_rt130_packet.EH_PAYLOAD = EH_PAYLOAD


class EHPacket(obspy_rt130_packet.EHPacket):
    def __init__(self, data):
        self._data = data
        payload = self._data["payload"].tobytes()
        for name, (start, length, converter) in EH_PAYLOAD.items():
            data = payload[start:start + length]
            if converter is not None:
                data = converter(data)
            setattr(self, name, data)

    def __str__(self, compact=False):
        if compact:
            sta = (self.station_name.strip() +
                   self.station_name_extension.strip())
            info = ("{:04d} {:2s} {:4s} {:2d} {:4d} {:4d} {:2d} {:2s} "
                    "{:5s}  {:4s}        {!s}").format(
                self.packet_sequence, self.type.decode(),
                self.unit_id.decode(), self.experiment_number,
                self.byte_count, self.event_number,
                self.data_stream_number, self.data_format.decode(),
                sta, str(self.sampling_rate)[:4], self.time)
        else:
            info = []
            for key in self._headers:
                value = getattr(self, key)
                if key in ("unit_id", "data_format"):
                    value = value.decode()
                info.append("{}: {}".format(key, value))
            info.append("-" * 20)
            for key in sorted(EH_PAYLOAD.keys()):
                value = getattr(self, key)
                if key in ("trigger_time", "detrigger_time",
                           "first_sample_time", "last_sample_time"):
                    if value is not None:
                        value = UTCDateTime(ns=value)
                info.append("{}: {}".format(key, value))
            info = "{} Packet\n\t{}".format(self.type.decode(),
                                            "\n\t".join(info))
        return info

    def eh_et_info(self, nbr_DT_samples):
        """
        Compile EH and ET info to write to log file.
        Returns list of strings.
        Formatting of strings is based on earlier version of rt2ms.
        """
        info = []
        # packet_tagline1 = ("\n\n{:s} exp {:02d} bytes {:04d} {:s} ID: {:s} "
        #                    "seq {:04d}".format(self.type.decode(),
        #                                        self.experiment_number,
        #                                        self.byte_count,
        #                                        Packet.time_tag(self.time),
        #                                        self.unit_id.decode(),
        #                                        self.packet_sequence))
        # info.append(packet_tagline1)
        # if self.type.decode('ASCII') == 'EH':
        #     nbr_DT_samples = 0
        #     info.append("\nEvent Header")
        # else:
        #     info.append("\nEvent Trailer")
        # info.append("\n  event = " + str(self.event_number))
        # info.append("\n  stream = " + str(self.data_stream_number + 1))
        # info.append("\n  format = " + self.data_format.decode('ASCII'))
        # info.append("\n  stream name = " + self.stream_name)
        # info.append("\n  sample rate = " + str(self.sampling_rate))
        # info.append("\n  trigger type = " + self.trigger_type)
        trigger_time = Packet.time_tag(UTCDateTime(ns=self.trigger_time))
        # info.append("\n  trigger time = " + trigger_time)
        first_sample_time = Packet.time_tag(UTCDateTime(ns=self.first_sample_time))  # noqa: E501
        # info.append("\n  first sample = " + first_sample_time)
        # if self.last_sample_time:
        #     info.append("\n  last sample = " + Packet.time_tag(UTCDateTime(ns=self.last_sample_time)))  # noqa: E501
        # info.append("\n  bit weights = " + " ".join([val for val in self.channel_adjusted_nominal_bit_weights if val]))  # noqa: E501
        # info.append("\n  true weights = " + " ".join([val for val in self.channel_true_bit_weights if val]))  # noqa: E501
        packet_tagline2 = ("\nDAS: {:s} EV: {:04d} DS: {:d} FST = {:s} TT = "
                           "{:s} NS: {:d} SPS: {:.1f} ETO: 0"
                           .format(self.unit_id.decode(),
                                   self.event_number,
                                   self.data_stream_number + 1,
                                   first_sample_time,
                                   trigger_time,
                                   nbr_DT_samples,
                                   self.sampling_rate))
        info.append(packet_tagline2)
        return info


